const DEFAULT_THRESHOLD: i16 = 50;

pub enum ScrollAction {
    Previous,
    Next,
    None,
}

pub struct ScrollTracker {
    x: i16,
    y: i16,
    threshold: i16,
}

impl ScrollTracker {
    pub fn new() -> Self {
        Self {
            x: 0,
            y: 0,
            threshold: DEFAULT_THRESHOLD,
        }
    }

    pub fn new_with_threshold(threshold: i16) -> Self {
        Self {
            x: 0,
            y: 0,
            threshold,
        }
    }

    // Returns (x, y)
    pub fn scroll(&mut self, scroll_x: i8, scroll_y: i8) -> (ScrollAction, ScrollAction) {
        // only care about the bigger one
        // and also reset the smaller one
        // so that we don't scroll diagonally accidentally
        if scroll_x.abs() > scroll_y.abs() {
            self.y = 0;
            (
                scroll_one(&mut self.x, self.threshold, scroll_x),
                ScrollAction::None,
            )
        } else {
            self.x = 0;
            (
                ScrollAction::None,
                scroll_one(&mut self.y, self.threshold, scroll_y),
            )
        }
    }
}

fn scroll_one(tracker: &mut i16, threshold: i16, delta: i8) -> ScrollAction {
    *tracker = tracker.saturating_add(delta.into());

    if *tracker < -threshold {
        *tracker = 0;

        ScrollAction::Previous
    } else if *tracker > threshold {
        *tracker = 0;

        ScrollAction::Next
    } else {
        ScrollAction::None
    }
}
